package cn.kdan.compdf.controller.v1;

import cn.kdan.compdf.base.R;
import cn.kdan.compdf.dto.DashboardStatisticsDTO;
import cn.kdan.compdf.dto.PlanStatisticsDTO;
import cn.kdan.compdf.entity.dto.AnalysisDataDTO;
import cn.kdan.compdf.entity.dto.DashboardDetailedDataDTO;
import cn.kdan.compdf.entity.dto.DashboardQueryDTO;
import cn.kdan.compdf.enums.AuthEnum;
import cn.kdan.compdf.exception.CommonException;
import cn.kdan.compdf.service.BackgroundConvertDataService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @author comPDF-Kit WPH 2022-10-09
 */
@RestController
@RequestMapping("/v1/convert-data")
@RequiredArgsConstructor
public class BackgroundConvertDataController {

    private final BackgroundConvertDataService backgroundConvertDataService;

    @Value("${background.dashboard.autograph}")
    private String DASHBOARD_SYNC_AUTOGRAPH;

    /**
     * 获取统计信息
     */
    @GetMapping("/getAnalysisData-byDay")
    public R<AnalysisDataDTO> getAnalysisData(@Validated DashboardQueryDTO dashboardQueryDTO, HttpServletRequest request) {
        dashboardQueryDTO.setTenantId(new Long(request.getHeader(AuthEnum.USER_TENANT_ID_HEADER)));
        AnalysisDataDTO analysisData = backgroundConvertDataService.getAnalysisData(dashboardQueryDTO);
        return R.ok(analysisData);
    }

    /**
     * 获取成功数时间节点数据集合
     */
    @GetMapping("/getSuccessfulRequestDetailedData")
    public R<List<DashboardDetailedDataDTO>> getSuccessfulRequestDetailedData(@Validated DashboardQueryDTO dashboardQueryDTO, HttpServletRequest request) {
        dashboardQueryDTO.setTenantId(new Long(request.getHeader(AuthEnum.USER_TENANT_ID_HEADER)));
        List<DashboardDetailedDataDTO> successfulRequestDetailedData = backgroundConvertDataService.getSuccessfulRequestDetailedData(dashboardQueryDTO);
        return R.ok(successfulRequestDetailedData);
    }

    /**
     * 获取失败数时间节点数据集合
     */
    @GetMapping("/getErrorRequestDetailedData")
    public R<List<DashboardDetailedDataDTO>> getErrorRequestDetailedData(@Validated DashboardQueryDTO dashboardQueryDTO, HttpServletRequest request) {
        dashboardQueryDTO.setTenantId(new Long(request.getHeader(AuthEnum.USER_TENANT_ID_HEADER)));
        List<DashboardDetailedDataDTO> errorRequestDetailedData = backgroundConvertDataService.getErrorRequestDetailedData(dashboardQueryDTO);
        return R.ok(errorRequestDetailedData);
    }

    /**
     * 获取失败率时间节点数据集合
     */
    @GetMapping("/getErrorRatioDetailedData")
    public R<List<DashboardDetailedDataDTO>> getErrorRatioDetailedData(@Validated DashboardQueryDTO dashboardQueryDTO, HttpServletRequest request) {
        dashboardQueryDTO.setTenantId(new Long(request.getHeader(AuthEnum.USER_TENANT_ID_HEADER)));
        List<DashboardDetailedDataDTO> errorRatioDetailedData = backgroundConvertDataService.getErrorRatioDetailedData(dashboardQueryDTO);
        return R.ok(errorRatioDetailedData);
    }

    /**
     * 获取转档时长时间节点数据集合
     */
    @GetMapping("/getAverageProcessTimeDetailedData")
    public R<List<DashboardDetailedDataDTO>> getAverageProcessTimeDetailedData(@Validated DashboardQueryDTO dashboardQueryDTO, HttpServletRequest request) {
        dashboardQueryDTO.setTenantId(new Long(request.getHeader(AuthEnum.USER_TENANT_ID_HEADER)));
        List<DashboardDetailedDataDTO> averageProcessTimeDetailedData = backgroundConvertDataService.getAverageProcessTimeDetailedData(dashboardQueryDTO);
        return R.ok(averageProcessTimeDetailedData);
    }

    @GetMapping("/export")
    public void exportDashboard(@Validated DashboardQueryDTO dashboardQueryDTO,HttpServletRequest request, HttpServletResponse response) {
        dashboardQueryDTO.setTenantId(new Long(request.getHeader(AuthEnum.USER_TENANT_ID_HEADER)));
        backgroundConvertDataService.dashboardExport(dashboardQueryDTO, response);
    }

    /**
     * dashboard 数据同步接收
     */
    @PostMapping("/dashboardsync")
    public R<Void> dashboardsync(@RequestBody List<DashboardStatisticsDTO> dashboardStatisticsDTOList, HttpServletRequest request) {
        if (!DASHBOARD_SYNC_AUTOGRAPH.equals(request.getHeader("autograph"))) {
            throw new CommonException("错误");
        }
        backgroundConvertDataService.dashboardDataSync(dashboardStatisticsDTOList);
        return R.ok();
    }

    @PostMapping("/refresh")
    public void test(HttpServletRequest request) {
        Long tenantId = new Long(request.getHeader(AuthEnum.USER_TENANT_ID_HEADER));
        backgroundConvertDataService.refresh(tenantId);
    }


//    /**
//     * 获取列表
//     */
//    @GetMapping("/list")
//    public R<IPage<BackgroundConvertData>> list(BackgroundConvertData query) {
//        IPage<BackgroundConvertData> page = backgroundConvertDataService.page(query);
//        return R.ok(page);
//    }
//
//    /**
//     * 根据id获取
//     */
//    @GetMapping("/{id}")
//    public R<BackgroundConvertData> info(@PathVariable("id") Long id) {
//        return R.ok(backgroundConvertDataService.getById(id));
//    }
//
//    /**
//     * 保存
//     */
//    @PostMapping
//    public R<Void> save(@RequestBody BackgroundConvertData backgroundConvertData) {
//        backgroundConvertDataService.save(backgroundConvertData);
//        return R.ok();
//    }
//
//    /**
//     * 修改
//     */
//    @PutMapping
//    public R<Void> update(@RequestBody BackgroundConvertData backgroundConvertData) {
//        backgroundConvertDataService.updateById(backgroundConvertData);
//        return R.ok();
//    }
//
//    /**
//     * 删除
//     */
//    @DeleteMapping("/{id}")
//    public R<Void> remove(@PathVariable("id") Long id) {
//        backgroundConvertDataService.removeById(id);
//        return R.ok();
//    }
}
