package cn.kdan.compdf.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.kdan.compdf.dto.PlanStatisticsDTO;
import cn.kdan.compdf.entity.BackgroundUserBalance;
import cn.kdan.compdf.mapper.BackgroundUserBalanceMapper;
import cn.kdan.compdf.service.BackgroundUserBalanceService;
import cn.kdan.compdf.service.BackgroundUserService;
import cn.kdan.compdf.vo.BackgroundUserBalanceVO;
import cn.kdan.compdf.vo.BackgroundUserPackageBalanceVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.assertj.core.util.Lists;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @author ComPDFKit-WPH 2022-10-14
 */
@Service
@Slf4j
public class BackgroundUserBalanceServiceImpl extends ServiceImpl<BackgroundUserBalanceMapper, BackgroundUserBalance> implements BackgroundUserBalanceService {


    private final StringRedisTemplate redisTemplate;

    private final BackgroundUserService userService;

    public BackgroundUserBalanceServiceImpl(StringRedisTemplate redisTemplate, BackgroundUserService userService) {
        this.redisTemplate = redisTemplate;
        this.userService = userService;
    }

    @Override
    public IPage<BackgroundUserBalance> page(BackgroundUserBalance query) {
        return this.page(new Page<>(query.getPage(), query.getSize()), Wrappers.query(query));
    }

    /**
     * 获取账户套餐余额
     *
     * @param userId 用户id
     * @return 账户套餐余额
     */
    @Override
    public BackgroundUserPackageBalanceVO getPackageBalance(Long userId) {
        LambdaQueryWrapper<BackgroundUserBalance> eq = new LambdaQueryWrapper<BackgroundUserBalance>().eq(BackgroundUserBalance::getUserId, userId);
        List<BackgroundUserBalance> backgroundUserBalanceList = this.baseMapper.selectList(eq);
        BackgroundUserPackageBalanceVO backgroundUserPackageBalanceVO = BackgroundUserPackageBalanceVO.builder().usedFiles(0).remainingFiles(0).build();
        if (CollectionUtil.isNotEmpty(backgroundUserBalanceList)) {
            backgroundUserBalanceList.forEach(c -> {
                // 1充值
                if (c.getChangeType() == 1) {
                    backgroundUserPackageBalanceVO.setRemainingFiles(backgroundUserPackageBalanceVO.getRemainingFiles() + c.getBalanceChange());
                }
                // 2正常消费扣除
                if (c.getChangeType() == 2) {
                    backgroundUserPackageBalanceVO.setUsedFiles(backgroundUserPackageBalanceVO.getUsedFiles() + c.getBalanceChange());
                }
            });
        }
        return backgroundUserPackageBalanceVO;
    }

    /**
     * 获取账户消费充值记录
     *
     * @param userId 用户id
     * @param page   当前页
     * @param size   当前页行数
     * @return 账户消费充值记录
     */
    @Override
    public IPage<BackgroundUserBalanceVO> getBalanceRecordList(Long userId, Integer page, Integer size) {
        LambdaQueryWrapper<BackgroundUserBalance> eq = new LambdaQueryWrapper<BackgroundUserBalance>()
                .eq(BackgroundUserBalance::getUserId, userId)
                .orderByDesc(BackgroundUserBalance::getCreateDate);
        Page<BackgroundUserBalance> backgroundUserBalancePage = this.baseMapper.selectPage(new Page<>(page, size), eq);
        // 处理返回数据
        Page<BackgroundUserBalanceVO> backgroundUserBalanceVOPage = new Page<>();
        backgroundUserBalanceVOPage.setCurrent(backgroundUserBalancePage.getCurrent());
        backgroundUserBalanceVOPage.setPages(backgroundUserBalancePage.getPages());
        backgroundUserBalanceVOPage.setSize(backgroundUserBalancePage.getSize());
        backgroundUserBalanceVOPage.setTotal(backgroundUserBalancePage.getTotal());
        List<BackgroundUserBalanceVO> records = Lists.newArrayList();
        if (CollectionUtil.isNotEmpty(backgroundUserBalancePage.getRecords())) {
            backgroundUserBalancePage.getRecords().forEach(c -> {
                BackgroundUserBalanceVO backgroundUserBalanceVO = new BackgroundUserBalanceVO();
                BeanUtil.copyProperties(c, backgroundUserBalanceVO);
                records.add(backgroundUserBalanceVO);
            });
        }
        backgroundUserBalanceVOPage.setRecords(records);
        return backgroundUserBalanceVOPage;
    }

    @Override
    public void planSync(List<PlanStatisticsDTO> list) {
        assert list != null : "Dashboard同步数据为空";
        final String PLAN_SYNC_KEY = "planSyncKey:" + list.get(0).getSyncTime();
        Boolean b = redisTemplate.hasKey(PLAN_SYNC_KEY);
        assert b != null;
        if (!b) {
            log.info("开始数据同步");
            redisTemplate.opsForValue().setIfAbsent(PLAN_SYNC_KEY, "", 1, TimeUnit.DAYS);
            try {
                List<BackgroundUserBalance> userBalanceList = new ArrayList<>();
                list.forEach(item -> {
                    Long userId = userService.getUserId(item.getTenantId());
                    BackgroundUserBalance backgroundUserBalance = new BackgroundUserBalance();
                    backgroundUserBalance.setUserId(userId);
                    backgroundUserBalance.setTenantId(item.getTenantId());
                    backgroundUserBalance.setDate(Date.from(item.getSyncTime().atZone(ZoneId.systemDefault()).toInstant()));
                    backgroundUserBalance.setDescriptionType(9);
                    backgroundUserBalance.setDescription("Files consumed");
                    backgroundUserBalance.setChangeType(2);
                    backgroundUserBalance.setBalanceChange(item.getPlan().intValue());
                    userBalanceList.add(backgroundUserBalance);
                });
                this.saveBatch(userBalanceList);
            }catch (Exception e){
                redisTemplate.delete(PLAN_SYNC_KEY);
                throw e;
            }
        } else {
            log.error("当前时间节点已同步过：{}" + PLAN_SYNC_KEY);
        }
    }

}
